import asyncio

from rofi_menu.constants import OP_EXIT, OP_OUTPUT
from rofi_menu.menu import Item, Operation


class ShellItem(Item):
    def __init__(self, text=None, command="echo OK", **kwargs):
        self._command = command
        self.show_output = kwargs.pop("show_output", False)
        self.detached = kwargs.pop("detached", True)
        super().__init__(text, **kwargs)

    @property
    def command(self):
        return self._command

    async def on_select(self, item_id, meta):
        command = f"nohup {self.command}" if self.detached else self.command
        proc = await asyncio.create_subprocess_shell(
            command,
            stdout=(
                asyncio.subprocess.PIPE
                if self.show_output
                else asyncio.subprocess.DEVNULL
            ),
            stderr=asyncio.subprocess.DEVNULL,
        )
        if self.show_output:
            data = await proc.stdout.read()
            return Operation(OP_OUTPUT, data.decode("utf-8"))
        return Operation(OP_EXIT)



from rofi_menu.menu import NestedMenu, Menu
from .shell import ShellItem


class SecondMonitorMenu(Menu):
    prompt = "Second monitor"
    secod_monitor_options = [
        {
            "text": '<span background="green"><b>Off</b></span>',
            "command": "xrandr --output {mon1} --auto --output {mon2} --off",
        },
        {
            "text": '<span background="blue"><b>Mirror</b></span>',
            "command": "xrandr --output {mon1} --auto --output {mon2} --auto --same-as {mon1}",
        },
        {
            "text": '<span background="gray"><b>Above</b></span>',
            "command": "xrandr --output {mon1} --auto --output {mon2} --auto --above {mon1}",
        },
        {
            "text": '<span background="gray"><b>Left</b></span>',
            "command": "xrandr --output {mon1} --auto --output {mon2} --auto --left-of {mon1}",
        },
        {
            "text": '<span background="gray"><b>Right</b></span>',
            "command": "xrandr --output {mon1} --auto --output {mon2} --auto --right-of {mon1}",
        },
        {
            "text": '<span background="gray"><b>Below</b></span>',
            "command": "xrandr --output {mon1} --auto --output {mon2} --auto --below {mon1}",
        },
    ]

    def __init__(
        self,
        prompt="Second monitor",
        primary_monitor="eDP1",
        secondary_monitor="HDMI1",
    ):
        items = [
            ShellItem(
                text=option["text"].format(
                    mon1=primary_monitor, mon2=secondary_monitor
                ),
                command=option["command"].format(
                    mon1=primary_monitor, mon2=secondary_monitor
                ),
            )
            for option in self.secod_monitor_options
        ]
        super().__init__(prompt=prompt, items=items)



import asyncio

from rofi_menu import constants, rofi_mode


class MetaStore:
    def __init__(self, selected_item):
        self.raw_input = selected_item
        self._meta = rofi_mode.parse_meta(selected_item)

    @property
    def selected_id(self):
        return self._meta.get("id")

    def get_state(self, item_id):
        return self._meta.get(".".join(item_id))

    def set_state(self, item_id, state):
        if state is not None:
            self._meta[".".join(item_id)] = state

    def as_dict(self):
        return self._meta


class Operation:
    def __init__(self, code, data=None):
        self.code = code
        self.data = data


class Item:
    icon = None
    text = None

    def __init__(self, text=None, *, flags=None):
        self.text = text or self.text
        self.flags = flags or set()

        # filled after attaching to menu
        self.id = None
        self.parent_menu = None
        self.state = None

        self.loaded = False

    def clone(self):
        obj = self.__class__()
        obj.__dict__.update(self.__dict__)
        return obj

    async def bind(self, menu, item_id, meta):
        """Link item to concreate menu, assign an id and return "bound" element."""
        obj = self.clone()
        obj.id = item_id
        obj.parent_menu = menu
        return obj

    async def load(self, meta):
        self.state = meta.get_state(self.id)

    async def store(self, meta):
        meta.set_state(self.id, self.state)

    async def pre_render(self, meta):
        if not self.loaded:
            await self.load(meta)

    async def render(self, meta):
        return self.text if self.text is not None else "[UNDEFINED]"

    async def post_render(self, meta):
        pass

    async def handle_render(self, meta):
        await self.pre_render(meta)
        text = await self.render(meta)
        await self.post_render(meta)
        return text

    async def pre_select(self, meta):
        if not self.loaded:
            await self.load(meta)

    async def on_select(self, item_id, meta):
        return Operation(constants.OP_REFRESH_MENU)

    async def post_select(self, meta):
        await self.store(meta)

    async def handle_select(self, item_id, meta):
        await self.pre_select(meta)
        op = await self.on_select(item_id, meta)
        await self.post_select(meta)
        return op


class BackItem(Item):
    text = ".."

    async def on_select(self, item_id, meta):
        return Operation(constants.OP_BACK_TO_PARENT_MENU)


class ExitItem(Item):
    async def on_select(self, item_id, meta):
        return Operation(constants.OP_EXIT)


class NestedMenu(Item):
    def __init__(self, text=None, menu=None, *, flags=None):
        super().__init__(text, flags=flags)
        self.sub_menu = menu or Menu()

    async def bind(self, menu, item_id, meta):
        """
        Link item to concreate menu, assign an id, initiate linking for submenu
        and return "bound" element.
        """
        obj = self.clone()
        obj.id = item_id
        obj.parent_menu = menu
        obj.sub_menu = await self.sub_menu.bind(prefix_path=item_id, meta=meta)
        return obj

    async def on_select(self, item_id, meta):
        if item_id == self.id:
            return Operation(constants.OP_OUTPUT, await self.sub_menu.render(meta))

        await self.sub_menu.pre_select(meta)
        op = await self.sub_menu.on_select(item_id, meta)
        await self.sub_menu.post_select(meta)

        if op.code == constants.OP_REFRESH_MENU:
            return Operation(constants.OP_OUTPUT, await self.sub_menu.render(meta))

        if op.code == constants.OP_BACK_TO_PARENT_MENU:
            return Operation(constants.OP_OUTPUT, await self.parent_menu.render(meta))

        return op


class Menu:
    prompt = "menu"
    items = ()

    def __init__(self, prompt=None, items=None):
        self.prompt = prompt or self.prompt
        self.items = items or self.items

    def clone(self):
        obj = self.__class__()
        obj.__dict__.update(self.__dict__)
        return obj

    async def bind(self, prefix_path, meta):
        """Link all nested items to the current menu and return "bound" element."""
        items = await self.generate_menu_items(prefix_path=prefix_path, meta=meta)
        # generate bound items
        obj = self.clone()
        obj.items = await asyncio.gather(*[
            item.bind(menu=obj, item_id=[*prefix_path, str(item_identity)], meta=meta)
            for item_identity, item in items
        ])
        return obj

    async def generate_menu_items(self, prefix_path, meta):
        return enumerate(self.items)

    async def pre_render(self, meta):
        pass

    async def render(self, meta):
        rendered_items = await asyncio.gather(
            *[item.handle_render(meta) for item in self.items]
        )

        _rofi_menu = []
        for num, item in enumerate(self.items):
            if constants.FLAG_STYLE_ACTIVE in item.flags:
                _rofi_menu.append(rofi_mode.menu_active(num))
            if constants.FLAG_STYLE_URGENT in item.flags:
                _rofi_menu.append(rofi_mode.menu_urgent(num))

        common_meta = meta.as_dict()
        _rofi_menu.extend(
            rofi_mode.menu_item(
                text=text,
                icon=item.icon,
                meta_data={**common_meta, "text": text, "id": item.id},
            )
            for text, item in zip(rendered_items, self.items)
        )

        return rofi_mode.render_menu(self.prompt, *_rofi_menu)

    async def post_render(self, meta):
        pass

    async def handle_render(self, meta):
        await self.pre_render(meta)
        text = await self.render(meta)
        await self.post_render(meta)
        return text

    async def pre_select(self, meta):
        pass

    async def on_select(self, item_id, meta):
        for item in self.items:
            if item.id == item_id[: len(item.id)]:
                op = await item.handle_select(item_id, meta)

                if op.code == constants.OP_REFRESH_MENU:
                    return Operation(constants.OP_OUTPUT, await self.render(meta))

                return op

        return Operation(constants.OP_OUTPUT, await self.render(meta))

    async def post_select(self, meta):
        pass

    async def handle_select(self, item_id, meta):
        await self.pre_select(meta)
        op = await self.on_select(item_id, meta)
        await self.post_select(meta)
        return op



import asyncio
import sys

from rofi_menu.constants import ROOT_MENU_ID, OP_EXIT, OP_OUTPUT
from rofi_menu.menu import MetaStore


def _output_menu(data):
    sys.stdout.write(data)


async def main(menu, selected_item):
    meta = MetaStore(selected_item)

    menu = await menu.bind(meta=meta, prefix_path=[ROOT_MENU_ID])

    if selected_item:
        op = await menu.handle_select(meta.selected_id, meta)

        if op.code == OP_OUTPUT:
            return _output_menu(op.data)

        elif op.code == OP_EXIT:
            exit(op.data or 1)

    _output_menu(await menu.handle_render(meta))


def run(menu):
    asyncio.run(main(menu, sys.argv[1] if len(sys.argv) > 1 else None))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rofi_menu', 'rofi_menu.contrib']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'rofi-menu',
    'version': '0.3.0',
    'description': 'Create rofi menus via python',
    'long_description': '<p align="center">\n    <a href="https://pypi.org/project/rofi-menu/">\n        <img src="https://badge.fury.io/py/rofi-menu.svg" alt="Package version">\n    </a>\n</p>\n\n# rofi-menu\n\nRofi allows defining custom modes ([see the spec](https://github.com/davatorium/rofi/wiki/mode-Specs)).\n\nThis lib is a reference implementation with some extra "sugar".\n\nSimple demo:\n\n![custom menu](https://github.com/miphreal/python-rofi-menu/raw/master/docs/demo.gif)\n\n## Installation\n\nUsing pip\n\n```sh\n$ pip install rofi-menu\n```\n\n## Example\n\nCreate a python script which will be used for rofi mode\ne.g. `example.py` (don\'t forget to mark it as executable -- `chmod +x ./example.py`)\n\nAssuming you installed `rofi-menu` into a virtual environment (let\'s say it\'s `~/.pyenv/versions/rofi/bin/python`).\nMake sure shebang points to the right python executable, e.g. `#!/home/user/pyenv/versions/rofi/bin/python`.\n```python\n#!/home/user/pyenv/versions/rofi/bin/python\nimport rofi_menu\n\n\nclass ProjectsMenu(rofi_menu.Menu):\n    prompt = "Projects"\n    items = [\n        rofi_menu.BackItem(),\n        rofi_menu.ShellItem("Project 1", "code-insiders ~/Develop/project1"),\n        rofi_menu.ShellItem("Project 2", "code-insiders ~/Develop/project2"),\n        rofi_menu.ShellItem("Project X", "code-insiders ~/Develop/projectx"),\n    ]\n\n\nclass LogoutMenu(rofi_menu.Menu):\n    prompt = "Logout"\n    items = [\n        rofi_menu.ShellItem("Yes", "i3-msg exit", flags={rofi_menu.FLAG_STYLE_URGENT}),\n        rofi_menu.ExitItem("No", flags={rofi_menu.FLAG_STYLE_ACTIVE}),\n    ]\n\n\nclass MainMenu(rofi_menu.Menu):\n    prompt = "menu"\n    items = [\n        rofi_menu.TouchpadItem(),\n        rofi_menu.NestedMenu("Projects >", ProjectsMenu()),\n        rofi_menu.ShellItem(\n            "Downloads (show size)", "du -csh ~/Downloads", show_output=True\n        ),\n        rofi_menu.NestedMenu("Second monitor", rofi_menu.SecondMonitorMenu()),\n        rofi_menu.ShellItem("Lock screen", "i3lock -i ~/.config/i3/bg.png"),\n        rofi_menu.ShellItem("Sleep", "systemctl suspend"),\n        rofi_menu.NestedMenu("Logout", LogoutMenu()),\n    ]\n\n\nif __name__ == "__main__":\n    rofi_menu.run(MainMenu())\n```\n\nRun it as:\n\n```sh\n$ rofi -modi mymenu:/path/to/example.py -show mymenu\n```\n\nIt\'ll result in\n\n![rofi menu](https://github.com/miphreal/python-rofi-menu/raw/master/docs/menu-example.png)',
    'author': 'miphreal',
    'author_email': 'miphreal@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/miphreal/python-rofi-menu',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import base64
import json
from typing import Optional

from rofi_menu.constants import MENU_ITEM_META_DELIM


def render_menu(prompt: str, *items) -> str:
    items = [menu_prompt(prompt), menu_enable_markup(), *items]
    return "\n".join(items)


def menu_prompt(text: str) -> str:
    return f"\x00prompt\x1f{text}"


def menu_enable_markup() -> str:
    return "\0markup-rows\x1ftrue"


def menu_message(text: str) -> str:
    return f"\0message\x1f{text}"


def menu_urgent(num: int) -> str:
    return f"\0urgent\x1f{num}"


def menu_active(num: int) -> str:
    return f"\0active\x1f{num}"


def menu_icon(text: str, icon: str) -> str:
    return f"{text}\0icon\x1f{icon}"


##
# Extending rofi
# Add meta data along all menu items


def menu_item(
    text: str, icon: Optional[str] = None, meta_data: Optional[dict] = None
) -> str:
    meta = json.dumps(meta_data)
    meta = base64.urlsafe_b64encode(meta.encode()).decode("utf8")
    text_with_meta = f"{text}{MENU_ITEM_META_DELIM}{meta}"
    if icon is not None:
        return menu_icon(text_with_meta, icon)
    return text_with_meta


def parse_meta(selected_item: str) -> dict:
    meta = None

    if isinstance(selected_item, str):
        meta = selected_item.partition(MENU_ITEM_META_DELIM)[-1]

    if meta:
        return json.loads(base64.urlsafe_b64decode(meta))
    return {}



import asyncio
import re

from rofi_menu.contrib.shell import ShellItem


class TouchpadItem(ShellItem):
    shell_cmd_device_id = (
        rf'$(xinput list | grep -Po "Touchpad.*id=\d+" | grep -Po "\d+")'
    )
    re_enabled_device = re.compile(r"Device Enabled.*:\t1")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.detached = False
        self.show_output = False

    @property
    def icon(self):
        return "input-touchpad-symbolic" if self.state else "touchpad-disabled-symbolic"

    @property
    def command(self):
        action = "enable" if self.state else "disable"
        return rf"xinput {action} " + self.shell_cmd_device_id

    async def load(self, meta):
        proc = await asyncio.create_subprocess_shell(
            rf"xinput list-props " + self.shell_cmd_device_id,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.DEVNULL,
        )
        data = (await proc.stdout.read()).decode("utf-8")
        self.state = bool(self.re_enabled_device.search(data))

    async def render(self, *args, **kwargs):
        state_on = '<span background="green"><b>ON</b></span>'
        state_off = '<span background="gray"><b>OFF</b></span>'
        return f"Touchpad [{self.state and state_on or state_off}]"

    async def on_select(self, *args, **kwargs):
        self.state = not self.state
        return await super().on_select(*args, **kwargs)



MENU_ITEM_META_DELIM = "\r!"

FLAG_STYLE_URGENT = "URGENT"
FLAG_STYLE_ACTIVE = "ACTIVE"

OP_OUTPUT = "OUTPUT_MENU"
OP_REFRESH_MENU = "REFRESH_MENU"
OP_BACK_TO_PARENT_MENU = "BACK_TO_PARENT_MENU"
OP_EXIT = "EXIT"

ROOT_MENU_ID = "root"
